﻿Imports System.Data.Entity.Validation
Imports System.Web.Mvc
Imports HowDoICodeFirst.FluentMappings.Models
Imports System.IO

Namespace HowDoICodeFirst.FluentMappings.Controllers
	Public Class PostController
		Inherits Controller
		Private db As New BlogContext()

		'
		' GET: /Post/

		Public Function Index(ByVal blogId As Integer) As ViewResult
			ViewBag.BlogId = blogId
			Return View(db.Posts.Where(Function(p) p.Blog.Id = blogId).ToList())
		End Function

		'
		' GET: /Post/Details/5

		Public Function Details(ByVal id As Integer) As ViewResult
			Dim post As Post = db.Posts.Find(id)
			Return View(post)
		End Function

		'
		' GET: /Post/Create

		Public Function Create(ByVal blogId As Integer) As ActionResult
			Return View()
		End Function

		'
		' POST: /Post/Create
		Public Function ShowImage(ByVal image As String) As FileContentResult
			Return File(Convert.FromBase64String(image), System.Net.Mime.MediaTypeNames.Image.Jpeg)

		End Function

		<HttpPost>
		Public Function Create(ByVal post As Post) As ActionResult

			Dim filePath = Me.Request.Form("fileToUpload")

			If Request.Files.Count = 1 Then
				Dim img = Request.Files(0)
				If img.ContentLength > 0 Then
					post.Image = ReadFully(img.InputStream)
				End If
			End If

			Try
				db.Posts.Add(post)
				db.SaveChanges()
				Return RedirectToAction("Index", New With {Key .blogId = post.BlogId})

			Catch ex As DbEntityValidationException
				Dim [error] = ex.EntityValidationErrors.First().ValidationErrors.First()
				Me.ModelState.AddModelError([error].PropertyName, [error].ErrorMessage)
				ViewBag.BlogId = New SelectList(db.Blogs, "Id", "Title", post.BlogId)
				Return View(post)
			End Try
		End Function




		'
		' GET: /Post/Edit/5


		Public Function Edit(ByVal id As Integer) As ActionResult
			Dim post As Post = db.Posts.Find(id)
			ViewBag.BlogId = New SelectList(db.Blogs, "Id", "Title", post.BlogId)
			Return View(post)
		End Function

		'
		' POST: /Post/Edit/5

		<HttpPost, ValidateInput(False)>
		Public Function Edit(ByVal post As Post) As ActionResult
			If ModelState.IsValid Then
				db.Entry(post).State = EntityState.Modified
				db.SaveChanges()
				Return RedirectToAction("Index", New With {Key .blogId = post.BlogId})
			End If
			ViewBag.BlogId = New SelectList(db.Blogs, "Id", "Title", post.BlogId)
			Return View(post)
		End Function

		'
		' GET: /Post/Delete/5

		Public Function Delete(ByVal id As Integer) As ActionResult
			Dim post As Post = db.Posts.Find(id)
			Return View(post)
		End Function

		'
		' POST: /Post/Delete/5

		<HttpPost, ActionName("Delete")>
		Public Function DeleteConfirmed(ByVal id As Integer) As ActionResult
			Dim post As Post = db.Posts.Find(id)
			Dim blogId = post.BlogId
			db.Posts.Remove(post)
			db.SaveChanges()
			Return RedirectToAction("Index", New With {Key .blogId = blogId})
		End Function

		Public Shared Function ReadFully(ByVal input As Stream) As Byte()
			Dim buffer(16 * 1024 - 1) As Byte
			Using ms As New MemoryStream()
				Dim read As Integer
				read = input.Read(buffer, 0, buffer.Length)
				Do While read > 0
					ms.Write(buffer, 0, read)
					read = input.Read(buffer, 0, buffer.Length)
				Loop
				Return ms.ToArray()
			End Using
		End Function



	End Class

	Public Module Extensions
		<System.Runtime.CompilerServices.Extension> _
		Public Function HasFile(ByVal file As HttpPostedFileBase) As Boolean
			Return If(file IsNot Nothing AndAlso file.ContentLength > 0, True, False)
		End Function
	End Module

End Namespace